SystemOrganization addCategory: #'Rest-Server-Handlers'!
SystemOrganization addCategory: #'Rest-Server-Services'!
SystemOrganization addCategory: #'Rest-Server'!

!HTTPPut methodsFor: '*rest-server' stamp: 'SR 2/24/2009 13:10'!
for: aRequestLine readFrom: aSwazooStream 
	| contentTypeField |
	super for: aRequestLine readFrom: aSwazooStream.

	contentTypeField := self headers fieldOfClass: ContentTypeField
		ifNone: [SwazooHTTPPutError raiseSignal: 'Missing Content-Type'].
	self octetDataFrom: aSwazooStream.! !

Object subclass: #RestHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestHttpHandler methodsFor: 'request processing' stamp: 'mt 2/16/2009 15:18'!
process: anHttpRequest
	"Handles an http request. Should answer with an http response."
	
	self subclassResponsibility! !

RestHttpHandler subclass: #RestResourceHttpHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestResourceHttpHandler class methodsFor: 'error handling' stamp: 'mfp 2/20/2009 15:37'!
responseForFailure
	^ HTTPResponse internalServerError.! !

!RestResourceHttpHandler methodsFor: 'parsing' stamp: 'cs 2/23/2009 18:26'!
pathComponents: anHTTPRequest
	"remove leading '/'"
	
	^ anHTTPRequest uri identifierPath allButFirst.! !

!RestResourceHttpHandler methodsFor: 'request processing' stamp: 'SR 2/24/2009 12:52'!
process: anHTTPRequest 
	"Create a resource chain from the path of the http request. Fail if that chain is empty. Redirect processing according to used verb (POST, PUT, GET, DELETE)."
	| processorClass |

	anHTTPRequest isGet ifTrue: [processorClass := RestGetRequestProcessor].
	anHTTPRequest isPost ifTrue: [processorClass := RestPostRequestProcessor].
	anHTTPRequest isDelete ifTrue: [processorClass := RestDeleteRequestProcessor].
	anHTTPRequest isPut ifTrue: [processorClass := RestPutRequestProcessor].
	[^ (processorClass newFrom: anHTTPRequest handler: self)
		process: (self rootComponent: anHTTPRequest ifAbsent: [^ self class responseForFailure])
		components: (self pathComponents: anHTTPRequest) allButFirst]
		on: RestResolveError do: [:error | ^ error asHTTPResponse].! !

!RestResourceHttpHandler methodsFor: 'parsing' stamp: 'cs 2/24/2009 18:29'!
rootComponent: anHTTPRequest ifAbsent: aBlock
	^ self class rootResources
		associationAt: ((self pathComponents: anHTTPRequest) at: 1 ifAbsent: aBlock)
		ifAbsent: aBlock! !

Object subclass: #RestRequestProcessor
	instanceVariableNames: 'request handler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

RestRequestProcessor subclass: #RestDeleteRequestProcessor
	instanceVariableNames: 'postData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestDeleteRequestProcessor methodsFor: 'resolving' stamp: 'SR 2/24/2009 11:12'!
resolveRoot: root components: components
	| entity |
	entity := super resolveId: root value all components: components.

	(entity respondsTo: #delete)
		ifTrue: [entity perform: #delete]
		ifFalse: [RestResolveError signal: 'Selector #delete not found.'].
	^ ''! !

RestRequestProcessor subclass: #RestGetRequestProcessor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestGetRequestProcessor methodsFor: 'converting' stamp: 'mt 2/23/2009 11:50'!
convertObject: anObject
	"Create a dictionary with properties for the given object."
	
	^ anObject class allInstVarNames
		inject: Dictionary new
		into: [:old :varName| old at: varName put: (anObject instVarNamed: varName); yourself].! !

!RestGetRequestProcessor methodsFor: 'processing' stamp: 'cs 2/23/2009 14:15'!
resolveMethod: anEntity components: pathComponents
	
	^ pathComponents isEmpty
		ifTrue: [^ self convertObject: anEntity]
		ifFalse: [^ super resolveMethod: anEntity components: pathComponents]! !

!RestGetRequestProcessor methodsFor: 'resolving' stamp: 'cs 2/23/2009 16:48'!
resolveRoot: root components: pathComponents
	^ Json render:
		(self resolveId: root value all components: pathComponents)! !

RestRequestProcessor subclass: #RestPostRequestProcessor
	instanceVariableNames: 'postData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestPostRequestProcessor methodsFor: 'adding' stamp: 'cs 2/23/2009 17:53'!
add: aString on: anObject
	"resolves the symbol and performs an add-operation on the given object with the postData"
	| selector |
	selector := self postData isDictionary
		ifTrue: ['addNew', aString capitalized]
		ifFalse: ['add', aString capitalized].
	(selector endsWith: 'ies') ifTrue: [selector := (selector truncateTo: selector size - 3), 'y'].
	(selector endsWith: 's') ifTrue: [selector := selector truncateTo: selector size - 1].
	selector := (selector, ':') asSymbol.
	
	(anObject respondsTo: selector)
		ifTrue: [^ (anObject perform: selector with: self postData) restUri]
		ifFalse: [RestResolveError signal: ('Selector {1} not found.' format: {selector})].
! !

!RestPostRequestProcessor methodsFor: 'as yet unclassified' stamp: 'cs 2/23/2009 18:25'!
postData
	postData ifNil: [ | data |
		data := self request ensureFullRead entityBody value.
		postData := (data startsWith: '{')
			ifTrue: [Json readFrom: data readStream]
			ifFalse: [self resolveUri: data]].
	^ postData.! !

!RestPostRequestProcessor methodsFor: 'resolving' stamp: 'cs 2/23/2009 16:44'!
resolveMethod: anEntity components: pathComponents
	^ pathComponents size = 1
		ifTrue: [self add: pathComponents first on: anEntity]
		ifFalse: [super resolveMethod: anEntity components: pathComponents]! !

!RestPostRequestProcessor methodsFor: 'resolving' stamp: 'cs 2/23/2009 16:45'!
resolveRoot: root components: components
	^ components isEmpty
		ifTrue: [self add: root key on: root value]
		ifFalse: [super resolveId: root value all components: components]! !

RestRequestProcessor subclass: #RestPutRequestProcessor
	instanceVariableNames: 'putData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestPutRequestProcessor methodsFor: 'as yet unclassified' stamp: 'SR 2/24/2009 12:49'!
putData
	putData ifNil: [ | data |
		data := (HTTPString stringFromBytes: self request putData) value.
		putData := (data startsWith: '{')
			ifTrue: [Json readFrom: data readStream]
			ifFalse: [RestResolveError signal: 'Invalid put data.']].
	^ putData.! !

!RestPutRequestProcessor methodsFor: 'resolving' stamp: 'SR 2/24/2009 12:51'!
resolveRoot: root components: pathComponents
	| entity selector |
	entity := self resolveId: root value all components: pathComponents.
	self putData keysAndValuesDo: [:key :value | 
		selector := (key, ':') asSymbol.
		(entity respondsTo: selector)
			ifTrue: [entity perform: selector with: value]
			ifFalse: [RestResolveError signal: ('Selector {1} not found.' format: {selector})]].
	^ ''! !

!RestRequestProcessor class methodsFor: 'instance creation' stamp: 'cs 2/23/2009 18:06'!
newFrom: anHTTPRequest handler: aHandler
	^ self new
		request: anHTTPRequest;
		handler: aHandler! !

!RestRequestProcessor methodsFor: 'accessing' stamp: 'cs 2/23/2009 18:06'!
handler
	^ handler! !

!RestRequestProcessor methodsFor: 'accessing' stamp: 'cs 2/23/2009 18:06'!
handler: anObject
	handler := anObject! !

!RestRequestProcessor methodsFor: 'processing' stamp: 'cs 2/23/2009 16:48'!
process: root components: pathComponents
	^ HTTPResponse ok entity:
		(self resolveRoot:root components: pathComponents).! !

!RestRequestProcessor methodsFor: 'accessing' stamp: 'cs 2/23/2009 16:08'!
request
	^ request! !

!RestRequestProcessor methodsFor: 'accessing' stamp: 'cs 2/23/2009 16:08'!
request: anObject
	request := anObject! !

!RestRequestProcessor methodsFor: 'resolving' stamp: 'cs 2/23/2009 13:26'!
resolveId: aCollection components: pathComponents
	| id resource |
	pathComponents ifEmpty: [^ aCollection].
	
	id := pathComponents first asNumber.
	resource := aCollection 
		detect: [:each | each id = id] 
		ifNone: [RestResolveError signal: ('Entity with id {1} not found.' format: {id})].

	^ self resolveMethod: resource components: pathComponents allButFirst! !

!RestRequestProcessor methodsFor: 'resolving' stamp: 'cs 2/23/2009 13:29'!
resolveMethod: anEntity components: pathComponents
	
	| selector resource |
	pathComponents ifEmpty: [^ anEntity].
	
	selector := pathComponents first asSymbol.
	(anEntity class allSelectors includes: selector)
		ifTrue: [resource := anEntity perform: selector]
		ifFalse: [RestResolveError signal: ('Selector {1} not found.' format: {selector})].

	^ self resolveId: resource components: pathComponents allButFirst! !

!RestRequestProcessor methodsFor: 'resolving' stamp: 'cs 2/23/2009 13:54'!
resolveRoot: root components: pathComponents
	self subclassResponsibility! !

!RestRequestProcessor methodsFor: 'helping' stamp: 'cs 2/23/2009 18:27'!
resolveUri: aString
	| newRequest |
	"Finds an object with the provided uri by using an pseudo-get-requet"
	newRequest := HTTPGet request: aString.
	^ (RestRequestProcessor newFrom: newRequest handler: self handler)
		resolveId: (self handler rootComponent: newRequest
			ifAbsent: [RestResolveError signal: 'invlaid URI given']) value all
		components: (self handler pathComponents: newRequest) allButFirst! !

!HttpRequest methodsFor: '*rest-server' stamp: 'mt 2/18/2009 14:21'!
putFields
	"Answer the multipart fields for a post request (if in fact this is a
	PUT request"

	^self propertyAt: #putFields ifAbsentPut: [
		" parse PUT fields in the body of request, if it's standard form data "

		(self isPutRequest and: [ 
			self contentType = MIMEDocument contentTypeFormData 
		]) ifTrue: [
			self class
				decodeUrlEncodedForm: (stream next: self contentLength)
				multipleValues: true.
		] ifFalse: [
			Dictionary new
		].
	]! !

Resource subclass: #RestHttpService
	instanceVariableNames: 'handlers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Services'!

RestHttpService class
	instanceVariableNames: 'site resource'!

!RestHttpService commentStamp: 'mt 2/16/2009 14:56' prior: 0!
I am a restful http service. I am primarily used for the XP integration platform project. (Package: 'Xpf')!

RestHttpService class
	instanceVariableNames: 'site resource'!

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/19/2009 16:05'!
isRunning
	^site notNil and: [site isServing]! !

!RestHttpService class methodsFor: 'service' stamp: 'cs 2/19/2009 17:51'!
port
	^site port! !

!RestHttpService class methodsFor: 'accessing' stamp: 'cs 2/24/2009 17:37'!
resource
	^ resource! !

!RestHttpService class methodsFor: 'accessing' stamp: 'cs 2/24/2009 17:29'!
resource: anObject
	^ resource := anObject! !

!RestHttpService class methodsFor: 'accessing' stamp: 'cs 2/24/2009 17:20'!
site
	^ site! !

!RestHttpService class methodsFor: 'accessing' stamp: 'cs 2/24/2009 17:20'!
site: anObject
	site := anObject! !

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/19/2009 16:08'!
start

	self startOn: 8082.

! !

!RestHttpService class methodsFor: 'service' stamp: 'topa 2/25/2009 10:49'!
startOn: aPort
	"Start exactly one http service. Does nothing if server is already running."

	self isRunning ifTrue: [^self].
	site := Site new
		name: 'rest';
		host: 'localhost' ip: '127.0.0.1' port: aPort.

	SwazooServer singleton addSite: site.
	
	resource := RestHttpService new uriPattern: '/'.
	site
		"start;"
		addResource: resource.
	SwazooServer singleton startSite: 'rest'.! !

!RestHttpService class methodsFor: 'service' stamp: 'mt 2/19/2009 17:15'!
stop
	"Shutdown the server. Does nothing if server is not running."

	site ifNotNil:
		[site stop.
		SwazooServer singleton removeSite: site.
		site := nil.
		resource := nil].! !

!RestHttpService methodsFor: 'serving' stamp: 'topa 2/23/2009 17:48'!
answerTo: request
	
	^ (handlers
		detect: [:ass| request uri asString startsWith: ass key]
		ifNone: [^ HTTPResponse notFound])
			value new process: request! !

!RestHttpService methodsFor: 'initialization' stamp: 'topa 2/23/2009 17:36'!
initialize
	super initialize.
	handlers := OrderedCollection new	"needed for priorities"! !

!RestHttpService methodsFor: 'serving' stamp: 'cs 2/15/2009 16:07'!
installHandler: aRequestHandlerClass on: aPathString
	(handlers anySatisfy: [:ass | ass key = aPathString ])
		ifTrue: [^ false].
	handlers add: aPathString -> aRequestHandlerClass! !

!Collection methodsFor: '*rest-server' stamp: 'cs 2/16/2009 00:38'!
jsonWriteOn: aStream
	| needComma |
	needComma := false.
	aStream nextPut: $[.
	self do: [:v |
		needComma
			ifTrue: [ aStream nextPutAll: ', ' ]
			ifFalse: [ needComma := true ].
		v jsonWriteOn: aStream.
	].
	aStream nextPut: $].! !

Error subclass: #RestResolveError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Server-Handlers'!

!RestResolveError methodsFor: 'as yet unclassified' stamp: 'mt 2/23/2009 11:48'!
asHTTPResponse
	^ HTTPResponse notFound
		entity: self messageText! !
